@@ -61,7 +61,9 @@ class Database {
                         starred     BOOL NOT NULL,
                         source      INT NOT NULL,
                         uid         VARCHAR(255) NOT NULL,
-                        link        TEXT NOT NULL
+                        link        TEXT NOT NULL,
+                        updatetime  DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP,
+                        author      VARCHAR(255)
                     );
                 ');
                 
@@ -70,6 +72,15 @@ class Database {
                         source
                     );
                 ');
+                \F3::get('db')->exec('
+                    CREATE TRIGGER update_updatetime_trigger
+                    AFTER UPDATE ON items FOR EACH ROW
+                        BEGIN
+                            UPDATE items
+                            SET updatetime = CURRENT_TIMESTAMP
+                            WHERE id = NEW.id;
+                        END;
+                 ');
             }
             
             $isNewestSourcesTable = false;
@@ -97,7 +108,7 @@ class Database {
                 ');
                 
                 \F3::get('db')->exec('
-                    INSERT INTO version (version) VALUES (3);
+                    INSERT INTO version (version) VALUES (5);
                 ');
                 
                 \F3::get('db')->exec('
@@ -117,7 +128,7 @@ class Database {
                 $version = @\F3::get('db')->exec('SELECT version FROM version ORDER BY version DESC LIMIT 0, 1');
                 $version = $version[0]['version'];
 
-                if($version == "2"){
+                if(strnatcmp($version, "3") < 0){
                     \F3::get('db')->exec('
                         ALTER TABLE sources ADD lastupdate INT;
                     ');
@@ -125,6 +136,40 @@ class Database {
                         INSERT INTO version (version) VALUES (3);
                     ');
                 }
+                if(strnatcmp($version, "4") < 0){
+                    \F3::get('db')->exec('
+                        ALTER TABLE items ADD updatetime DATETIME;
+                    ');
+                    \F3::get('db')->exec('
+                        CREATE TRIGGER insert_updatetime_trigger
+                        AFTER INSERT ON items FOR EACH ROW
+                            BEGIN
+                                UPDATE items
+                                SET updatetime = CURRENT_TIMESTAMP
+                                WHERE id = NEW.id;
+                            END;
+                    ');
+                    \F3::get('db')->exec('
+                        CREATE TRIGGER update_updatetime_trigger
+                        AFTER UPDATE ON items FOR EACH ROW
+                            BEGIN
+                                UPDATE items
+                                SET updatetime = CURRENT_TIMESTAMP
+                                WHERE id = NEW.id;
+                            END;
+                    ');
+                    \F3::get('db')->exec('
+                        INSERT INTO version (version) VALUES (4);
+                    ');
+                }
+                if(strnatcmp($version, "5") < 0){
+                    \F3::get('db')->exec('
+                        ALTER TABLE items ADD author VARCHAR(255);
+                    ');
+                    \F3::get('db')->exec('
+                        INSERT INTO version (version) VALUES (5);
+                    ');
+                }
             }
             
             // just initialize once
