@@ -123,15 +123,27 @@ class ContentLoader {
             // insert new item
             \F3::get('logger')->log('start insertion of new item "'.$item->getTitle().'"', \DEBUG);
             
-            // sanitize content html
-            $content = $this->sanitizeContent($item->getContent());
+            $content = "";
+            try {
+                // fetch content
+                $content = $item->getContent();
+                
+                // sanitize content html
+                $content = $this->sanitizeContent($content);
+            } catch(\exception $e) {
+                $content = 'Error: Content not fetched. Reason: ' . $e->getMessage();
+                \F3::get('logger')->log('Can not fetch "'.$item->getTitle().'" : ' . $e->getMessage(), \ERROR);
+            }
 
             // sanitize title
-            $title = htmlspecialchars_decode($item->getTitle());
-            $title = htmLawed($title, array("deny_attribute" => "*", "elements" => "-*"));
+            $title = $this->sanitizeContent($item->getTitle());
             if(strlen(trim($title))==0)
                 $title = "[" . \F3::get('lang_no_title') . "]";
 
+            // sanitize author
+            $author = htmlspecialchars_decode($item->getAuthor());
+            $author = htmLawed($author, array("deny_attribute" => "*", "elements" => "-*"));
+
             \F3::get('logger')->log('item content sanitized', \DEBUG);
 
             try {
@@ -148,7 +160,8 @@ class ContentLoader {
                     'uid'          => $item->getId(),
                     'thumbnail'    => $item->getThumbnail(),
                     'icon'         => $icon!==false ? $icon : "",
-                    'link'         => htmLawed($item->getLink(), array("deny_attribute" => "*", "elements" => "-*"))
+                    'link'         => htmLawed($item->getLink(), array("deny_attribute" => "*", "elements" => "-*")),
+                    'author'       => $author
             );
             
             // save thumbnail
@@ -185,11 +198,11 @@ class ContentLoader {
             htmlspecialchars_decode($content),
             array(
                 "safe"           => 1,
-                "deny_attribute" => '* -alt -title -src -href',
+                "deny_attribute" => '* -alt -title -src -href -target',
                 "keep_bad"       => 0,
                 "comment"        => 1,
                 "cdata"          => 1,
-                "elements"       => 'div,p,ul,li,a,img,dl,dt,dd,h1,h2,h3,h4,h5,h6,ol,br,table,tr,td,blockquote,pre,ins,del,th,thead,tbody,b,i,strong,em,tt'
+                "elements"       => 'div,p,ul,li,a,img,dl,dt,dd,h1,h2,h3,h4,h5,h6,ol,br,table,tr,td,blockquote,pre,ins,del,th,thead,tbody,b,i,strong,em,tt,sub,sup'
             )
         );
     }
@@ -205,7 +218,7 @@ class ContentLoader {
     protected function fetchThumbnail($thumbnail, $newItem) {
         if (strlen(trim($thumbnail)) > 0) {
             $imageHelper = new \helpers\Image();
-            $thumbnailAsPng = $imageHelper->loadImage($thumbnail, 150, 150);
+            $thumbnailAsPng = $imageHelper->loadImage($thumbnail, 500, 500);
             if ($thumbnailAsPng !== false) {
                 file_put_contents(
                     'data/thumbnails/' . md5($thumbnail) . '.png',
