@@ -1,7 +1,7 @@
 <?php
 
 /*
-	Copyright (c) 2009-2013 F3::Factory/Bong Cosca, All rights reserved.
+	Copyright (c) 2009-2014 F3::Factory/Bong Cosca, All rights reserved.
 
 	This file is part of the Fat-Free Framework (http://fatfree.sf.net).
 
@@ -13,13 +13,32 @@
 	Please see the license.txt file for more information.
 */
 
+//! Factory class for single-instance objects
+abstract class Prefab {
+
+	/**
+	*	Return class instance
+	*	@return static
+	**/
+	static function instance() {
+		if (!Registry::exists($class=get_called_class())) {
+			$ref=new Reflectionclass($class);
+			$args=func_get_args();
+			Registry::set($class,
+				$args?$ref->newinstanceargs($args):new $class);
+		}
+		return Registry::get($class);
+	}
+
+}
+
 //! Base structure
-final class Base {
+class Base extends Prefab {
 
 	//@{ Framework details
 	const
 		PACKAGE='Fat-Free Framework',
-		VERSION='3.2.1-Dev';
+		VERSION='3.3.0-Release';
 	//@}
 
 	//@{ HTTP status codes (RFC 2616)
@@ -89,6 +108,7 @@ final class Base {
 		E_Fatal='Fatal error: %s',
 		E_Open='Unable to open %s',
 		E_Routes='No routes specified',
+		E_Class='Invalid class %s',
 		E_Method='Invalid method %s',
 		E_Hive='Invalid hive key %s';
 	//@}
@@ -100,12 +120,8 @@ final class Base {
 		$init,
 		//! Language lookup sequence
 		$languages,
-		//! Equivalent Locales
-		$locales,
 		//! Default fallback language
-		$fallback='en',
-		//! NULL reference
-		$null=NULL;
+		$fallback='en';
 
 	/**
 	*	Sync PHP global with corresponding hive key
@@ -129,10 +145,15 @@ final class Base {
 	/**
 	*	Replace tokenized URL with current route's token values
 	*	@return string
-	*	@param $url string
+	*	@param $url array|string
 	**/
 	function build($url) {
-		if (preg_match_all('/@(\w+)/',$url,$matches,PREG_SET_ORDER))
+		if (is_array($url))
+			foreach ($url as &$var) {
+				$var=$this->build($var);
+				unset($var);
+			}
+		elseif (preg_match_all('/@(\w+)/',$url,$matches,PREG_SET_ORDER))
 			foreach ($matches as $match)
 				if (array_key_exists($match[1],$this->hive['PARAMS']))
 					$url=str_replace($match[0],
@@ -152,6 +173,33 @@ final class Base {
 			$this->hive['PARAMS'][$pair[1]]=trim($pair[2]);
 	}
 
+	/**
+	*	Convert JS-style token to PHP expression
+	*	@return string
+	*	@param $str string
+	**/
+	function compile($str) {
+		$fw=$this;
+		return preg_replace_callback(
+			'/(?<!\w)@(\w(?:[\w\.\[\]]|\->|::)*)/',
+			function($var) use($fw) {
+				return '$'.preg_replace_callback(
+					'/\.(\w+)|\[((?:[^\[\]]*|(?R))*)\]/',
+					function($expr) use($fw) {
+						return '['.var_export(
+							isset($expr[2])?
+								$fw->compile($expr[2]):
+								(ctype_digit($expr[1])?
+									(int)$expr[1]:
+									$expr[1]),TRUE).']';
+					},
+					$var[1]
+				);
+			},
+			$str
+		);
+	}
+
 	/**
 	*	Get hive key reference/contents; Add non-existent hive keys,
 	*	array elements, and object properties by default
@@ -177,32 +225,32 @@ final class Base {
 				$obj=TRUE;
 			elseif ($obj) {
 				$obj=FALSE;
-				if ($add) {
-					if (!is_object($var))
-						$var=new stdclass;
+				if (!is_object($var))
+					$var=new stdclass;
+				if ($add || property_exists($var,$part))
 					$var=&$var->$part;
+				else {
+					$var=&$this->null;
+					break;
 				}
-				elseif (isset($var->$part))
-					$var=$var->$part;
-				else
-					return $this->null;
 			}
-			elseif ($add) {
+			else {
 				if (!is_array($var))
 					$var=array();
-				$var=&$var[$part];
+				if ($add || array_key_exists($part,$var))
+					$var=&$var[$part];
+				else {
+					$var=&$this->null;
+					break;
+				}
 			}
-			elseif (is_array($var) && isset($var[$part]))
-				$var=$var[$part];
-			else
-				return $this->null;
 		if ($parts[0]=='ALIASES')
 			$var=$this->build($var);
 		return $var;
 	}
 
 	/**
-	*	Return TRUE if hive key is not set
+	*	Return TRUE if hive key is set
 	*	(or return timestamp and TTL if cached)
 	*	@return bool
 	*	@param $key string
@@ -212,8 +260,7 @@ final class Base {
 		$val=$this->ref($key,FALSE);
 		return isset($val)?
 			TRUE:
-			(Cache::instance()->exists($this->hash($key).'.var',$val)?
-				$val:FALSE);
+			(Cache::instance()->exists($this->hash($key).'.var',$val)?:FALSE);
 	}
 
 	/**
@@ -243,8 +290,7 @@ final class Base {
 				$jar=$this->hive['JAR'];
 				if ($ttl)
 					$jar['expire']=time()+$ttl;
-				call_user_func_array('setcookie',
-					array_merge(array($parts[1],$val),$jar));
+				call_user_func_array('setcookie',array($parts[1],$val)+$jar);
 			}
 		}
 		else switch ($key) {
@@ -276,10 +322,6 @@ final class Base {
 		if (preg_match('/^JAR\b/',$key))
 			call_user_func_array(
 				'session_set_cookie_params',$this->hive['JAR']);
-		elseif (preg_match('/^SESSION\b/',$key)) {
-			session_commit();
-			session_start();
-		}
 		$cache=Cache::instance();
 		if ($cache->exists($hash=$this->hash($key).'.var') || $ttl)
 			// Persist the key-value pair
@@ -345,19 +387,7 @@ final class Base {
 			// Reset global to default value
 			$this->hive[$parts[0]]=$this->init[$parts[0]];
 		else {
-			$out='';
-			$obj=FALSE;
-			foreach ($parts as $part)
-				if ($part=='->')
-					$obj=TRUE;
-				elseif ($obj) {
-					$obj=FALSE;
-					$out.='->'.$out;
-				}
-				else
-					$out.='['.$this->stringify($part).']';
-			// PHP can't unset a referenced variable
-			eval('unset($this->hive'.$out.');');
+			eval('unset('.$this->compile('@this->hive.'.$key).');');
 			if ($parts[0]=='SESSION') {
 				session_commit();
 				session_start();
@@ -395,8 +425,8 @@ final class Base {
 	*	@param $dst string
 	**/
 	function copy($src,$dst) {
-		$ref=&$this->ref($dst,TRUE);
-		return $ref=$this->ref($src);
+		$ref=&$this->ref($dst);
+		return $ref=$this->ref($src,FALSE);
 	}
 
 	/**
@@ -406,7 +436,7 @@ final class Base {
 	*	@param $val string
 	**/
 	function concat($key,$val) {
-		$ref=&$this->ref($key,TRUE);
+		$ref=&$this->ref($key);
 		$ref.=$val;
 		return $ref;
 	}
@@ -418,7 +448,7 @@ final class Base {
 	*	@public
 	**/
 	function flip($key) {
-		$ref=&$this->ref($key,TRUE);
+		$ref=&$this->ref($key);
 		return $ref=array_combine(array_values($ref),array_keys($ref));
 	}
 
@@ -429,7 +459,7 @@ final class Base {
 	*	@param $val mixed
 	**/
 	function push($key,$val) {
-		$ref=&$this->ref($key,TRUE);
+		$ref=&$this->ref($key);
 		array_push($ref,$val);
 		return $val;
 	}
@@ -440,7 +470,7 @@ final class Base {
 	*	@param $key string
 	**/
 	function pop($key) {
-		$ref=&$this->ref($key,TRUE);
+		$ref=&$this->ref($key);
 		return array_pop($ref);
 	}
 
@@ -451,7 +481,7 @@ final class Base {
 	*	@param $val mixed
 	**/
 	function unshift($key,$val) {
-		$ref=&$this->ref($key,TRUE);
+		$ref=&$this->ref($key);
 		array_unshift($ref,$val);
 		return $val;
 	}
@@ -462,7 +492,7 @@ final class Base {
 	*	@param $key string
 	**/
 	function shift($key) {
-		$ref=&$this->ref($key,TRUE);
+		$ref=&$this->ref($key);
 		return array_shift($ref);
 	}
 
@@ -470,11 +500,11 @@ final class Base {
 	*	Merge array with hive array variable
 	*	@return array
 	*	@param $key string
-	*	@param $src array
+	*	@param $src string|array
 	**/
 	function merge($key,$src) {
-		$ref=&$this->ref($key,TRUE);
-		return array_merge($ref,$src);
+		$ref=&$this->ref($key);
+		return array_merge($ref,is_string($src)?$this->hive[$src]:$src);
 	}
 
 	/**
@@ -500,33 +530,34 @@ final class Base {
 	*	Convert PHP expression/value to compressed exportable string
 	*	@return string
 	*	@param $arg mixed
-	*	@param $detail bool
+	*	@param $stack array
 	**/
-	function stringify($arg,$detail=TRUE) {
-		$fw=$this;
-		$func=function($arg,$val) use($detail,$fw) {
-			return $arg===$val && !is_scalar($val)?
-				'*RECURSION*':$fw->stringify($val,$detail);
-		};
+	function stringify($arg,array $stack=NULL) {
+		if ($stack) {
+			foreach ($stack as $node)
+				if ($arg===$node)
+					return '*RECURSION*';
+		}
+		else
+			$stack=array();
 		switch (gettype($arg)) {
 			case 'object':
 				$str='';
-				if (!preg_match('/Base|Closure/',get_class($arg)) && $detail)
-					foreach ((array)$arg as $key=>$val)
-						$str.=($str?',':'').$this->stringify(
-							preg_replace('/[\x00].+?[\x00]/','',$key)).'=>'.
-							$func($arg,$val);
-				return method_exists($arg,'__tostring')?
-					$arg:
-					(addslashes(get_class($arg)).'::__set_state('.$str.')');
+				foreach (get_object_vars($arg) as $key=>$val)
+					$str.=($str?',':'').
+						var_export($key,TRUE).'=>'.
+						$this->stringify($val,
+							array_merge($stack,array($arg)));
+				return get_class($arg).'::__set_state(array('.$str.'))';
 			case 'array':
 				$str='';
 				$num=isset($arg[0]) &&
 					ctype_digit(implode('',array_keys($arg)));
 				foreach ($arg as $key=>$val)
 					$str.=($str?',':'').
-						($num?'':($this->stringify($key).'=>')).
-						$func($arg,$val);
+						($num?'':(var_export($key,TRUE).'=>')).
+						$this->stringify($val,
+							array_merge($stack,array($arg)));
 				return 'array('.$str.')';
 			default:
 				return var_export($arg,TRUE);
@@ -603,8 +634,8 @@ final class Base {
 	*	@param $str string
 	**/
 	function encode($str) {
-		return @htmlentities($str,ENT_COMPAT,$this->hive['ENCODING'],FALSE)?:
-			$this->scrub($str);
+		return @htmlentities($str,$this->hive['BITMASK'],
+			$this->hive['ENCODING'],FALSE)?:$this->scrub($str);
 	}
 
 	/**
@@ -613,29 +644,65 @@ final class Base {
 	*	@param $str string
 	**/
 	function decode($str) {
-		return html_entity_decode($str,ENT_COMPAT,$this->hive['ENCODING']);
+		return html_entity_decode($str,$this->hive['BITMASK'],
+			$this->hive['ENCODING']);
+	}
+
+	/**
+	*	Invoke callback recursively for all data types
+	*	@return mixed
+	*	@param $arg mixed
+	*	@param $func callback
+	*	@param $stack array
+	**/
+	function recursive($arg,$func,$stack=NULL) {
+		if ($stack) {
+			foreach ($stack as $node)
+				if ($arg===$node)
+					return $arg;
+		}
+		else
+			$stack=array();
+		switch (gettype($arg)) {
+			case 'object':
+				if (method_exists('ReflectionClass','iscloneable')) {
+					$ref=new ReflectionClass($arg);
+					if ($ref->iscloneable()) {
+						$arg=clone($arg);
+						foreach (get_object_vars($arg) as $key=>$val)
+							$arg->$key=$this->recursive($val,$func,
+								array_merge($stack,array($arg)));
+					}
+				}
+				return $arg;
+			case 'array':
+				$tmp=array();
+				foreach ($arg as $key=>$val)
+					$tmp[$key]=$this->recursive($val,$func,
+						array_merge($stack,array($arg)));
+				return $tmp;
+		}
+		return $func($arg);
 	}
 
 	/**
 	*	Remove HTML tags (except those enumerated) and non-printable
 	*	characters to mitigate XSS/code injection attacks
 	*	@return mixed
-	*	@param $var mixed
+	*	@param $arg mixed
 	*	@param $tags string
 	**/
-	function clean($var,$tags=NULL) {
-		if (is_string($var) && strlen($var)) {
-			if ($tags!='*')
-				$var=strip_tags($var,
-					'<'.implode('><',$this->split($tags)).'>');
-			$var=trim(preg_replace('/[\x00-\x08\x0B\x0C\x0E-\x1F]/','',$var));
-		}
-		elseif (is_array($var))
-			foreach ($var as &$val) {
-				$val=$this->clean($val,$tags);
-				unset($val);
+	function clean($arg,$tags=NULL) {
+		$fw=$this;
+		return $this->recursive($arg,
+			function($val) use($fw,$tags) {
+				if ($tags!='*')
+					$val=trim(strip_tags($val,
+						'<'.implode('><',$fw->split($tags)).'>'));
+				return trim(preg_replace(
+					'/[\x00-\x08\x0B\x0C\x0E-\x1F]/','',$val));
 			}
-		return $var;
+		);
 	}
 
 	/**
@@ -655,7 +722,6 @@ final class Base {
 	function format() {
 		$args=func_get_args();
 		$val=array_shift($args);
-		setlocale(LC_ALL,str_replace('-','_',$this->locales));
 		// Get formatting rules
 		$conv=localeconv();
 		return preg_replace_callback(
@@ -740,11 +806,15 @@ final class Base {
 								}
 							break;
 						case 'date':
-							return strftime(empty($mod) ||
-								$mod=='short'?'%x':'%A, %d %B %Y',
-								$args[$pos]);
+							if (empty($mod) || $mod=='short')
+								$prop='%x';
+							elseif ($mod=='long')
+								$prop='%A, %d %B %Y';
+							return strftime($prop,$args[$pos]);
 						case 'time':
-							return strftime('%X',$args[$pos]);
+							if (empty($mod) || $mod=='short')
+								$prop='%X';
+							return strftime($prop,$args[$pos]);
 						default:
 							return $expr[0];
 					}
@@ -775,7 +845,7 @@ final class Base {
 			}
 		}
 		$this->languages=array_unique($this->languages);
-		$this->locales=array();
+		$locales=array();
 		$windows=preg_match('/^win/i',PHP_OS);
 		foreach ($this->languages as $locale) {
 			if ($windows) {
@@ -785,9 +855,10 @@ final class Base {
 					$country=@constant('ISO::CC_'.strtolower($parts[1])))
 					$locale.='-'.$country;
 			}
-			$this->locales[]=$locale;
-			$this->locales[]=$locale.'.'.ini_get('default_charset');
+			$locales[]=$locale;
+			$locales[]=$locale.'.'.ini_get('default_charset');
 		}
+		setlocale(LC_ALL,str_replace('-','_',$locales));
 		return implode(',',$this->languages);
 	}
 
@@ -806,7 +877,6 @@ final class Base {
 			elseif (is_file($file=$base.'.ini')) {
 				preg_match_all(
 					'/(?<=^|\n)(?:'.
-					'(?:;[^\n]*)|(?:<\?php.+?\?>?)|'.
 					'(.+?)\h*=\h*'.
 					'((?:\\\\\h*\r?\n|.+?)*)'.
 					')(?=\r?\n|$)/',
@@ -870,7 +940,7 @@ final class Base {
 	function expire($secs=0) {
 		if (PHP_SAPI!='cli') {
 			header('X-Content-Type-Options: nosniff');
-			header('X-Frame-Options: SAMEORIGIN');
+			header('X-Frame-Options: '.$this->hive['XFRAME']);
 			header('X-Powered-By: '.$this->hive['PACKAGE']);
 			header('X-XSS-Protection: 1; mode=block');
 			if ($secs) {
@@ -927,13 +997,12 @@ final class Base {
 		// Analyze stack trace
 		foreach ($trace as $frame) {
 			$line='';
-			if ($debug>1) {
-				if (isset($frame['class']))
-					$line.=$frame['class'].$frame['type'];
-				if (isset($frame['function']))
-					$line.=$frame['function'].'('.(isset($frame['args'])?
+			if (isset($frame['class']))
+				$line.=$frame['class'].$frame['type'];
+			if (isset($frame['function']))
+				$line.=$frame['function'].'('.
+					($debug>2 && isset($frame['args'])?
 						$this->csv($frame['args']):'').')';
-			}
 			$src=$this->fixslashes(str_replace($_SERVER['DOCUMENT_ROOT'].
 				'/','',$frame['file'])).':'.$frame['line'].' ';
 			error_log('- '.$src.$line);
@@ -947,8 +1016,6 @@ final class Base {
 			'text'=>$text,
 			'trace'=>$trace
 		);
-		if (!$debug && ob_get_level())
-			ob_end_clean();
 		$handler=$this->hive['ONERROR'];
 		$this->hive['ONERROR']=NULL;
 		if ((!$handler ||
@@ -987,10 +1054,8 @@ final class Base {
 			'(?:\h+\[('.implode('|',$types).')\])?/',$pattern,$parts);
 		$verb=strtoupper($parts[1]);
 		if ($parts[2]) {
-			if (empty($this->hive['ALIASES'][$parts[2]])) {
-				var_dump($parts);
+			if (empty($this->hive['ALIASES'][$parts[2]]))
 				user_error(sprintf(self::E_Named,$parts[2]));
-			}
 			$parts[4]=$this->hive['ALIASES'][$parts[2]];
 			if (isset($parts[3]))
 				$this->parse($parts[3]);
@@ -1008,7 +1073,7 @@ final class Base {
 			parse_str($query,$GLOBALS['_REQUEST']);
 		}
 		foreach ($headers?:array() as $key=>$val)
-			$_SERVER['HTTP_'.str_replace('-','_',strtoupper($key))]=$val;
+			$_SERVER['HTTP_'.strtr(strtoupper($key),'-','_')]=$val;
 		$this->hive['VERB']=$verb;
 		$this->hive['URI']=$this->hive['BASE'].$url['path'];
 		$this->hive['AJAX']=isset($parts[5]) &&
@@ -1080,8 +1145,7 @@ final class Base {
 			else
 				$url=$this->hive['BASE'].$url;
 			header('Location: '.$url);
-			$this->status($permanent?
-				301:($_SERVER['SERVER_PROTOCOL']<'HTTP/1.1'?302:303));
+			$this->status($permanent?301:302);
 			die;
 		}
 		$this->mock('GET '.$url);
@@ -1101,13 +1165,26 @@ final class Base {
 				$this->map($item,$class,$ttl,$kbps);
 			return;
 		}
-		$fluid=preg_match('/@\w+/',$class);
 		foreach (explode('|',self::VERBS) as $method)
-			if ($fluid ||
-				method_exists($class,$method) ||
-				method_exists($class,'__call'))
-				$this->route($method.' '.
-					$url,$class.'->'.strtolower($method),$ttl,$kbps);
+			$this->route($method.' '.$url,
+				$class.'->'.strtolower($method),$ttl,$kbps);
+	}
+
+	/**
+	*	Redirect a route to another URL
+	*	@return NULL
+	*	@param $pattern string|array
+	*	@param $url string
+	*/
+	function redirect($pattern,$url) {
+		if (is_array($pattern)) {
+			foreach ($pattern as $item)
+				$this->redirect($item,$url);
+			return;
+		}
+		$this->route($pattern,function($this) use ($url) {
+			$this->reroute($url);
+		});
 	}
 
 	/**
@@ -1157,7 +1234,7 @@ final class Base {
 			$url=str_replace("\x00".'@','@',$url);
 			if (!preg_match('/^'.
 				preg_replace('/@(\w+\b)/','(?P<\1>[^\/\?]+)',
-				str_replace('\*','(.*)',preg_quote($url,'/'))).
+				str_replace('\*','([^\?]*)',preg_quote($url,'/'))).
 				'\/?(?:\?.*)?$/'.$case.'um',$req,$args))
 				continue;
 			$route=NULL;
@@ -1179,7 +1256,7 @@ final class Base {
 					foreach (array_keys($args) as $key)
 						if (is_numeric($key) && $key)
 							unset($args[$key]);
-				if (is_string($handler))
+				if (is_string($handler)) {
 					// Replace route pattern tokens in handler if any
 					$handler=preg_replace_callback('/@(\w+\b)/',
 						function($id) use($args) {
@@ -1187,6 +1264,10 @@ final class Base {
 						},
 						$handler
 					);
+					if (preg_match('/(.+)\h*(?:->|::)/',$handler,$match) &&
+						!class_exists($match[1]))
+						$this->error(404);
+				}
 				// Capture values of route pattern tokens
 				$this->hive['PARAMS']=$args=array_map('urldecode',$args);
 				// Save matching route
@@ -1216,7 +1297,7 @@ final class Base {
 				else
 					$this->expire(0);
 				if (!strlen($body)) {
-					if (!$this->hive['RAW'])
+					if (!$this->hive['RAW'] && !$this->hive['BODY'])
 						$this->hive['BODY']=file_get_contents('php://input');
 					ob_start();
 					// Call route handler
@@ -1274,16 +1355,21 @@ final class Base {
 			preg_match('/(.+)\h*(->|::)\h*(.+)/s',$func,$parts)) {
 			// Convert string to executable PHP callback
 			if (!class_exists($parts[1]))
-				$this->error(404);
+				user_error(sprintf(self::E_Class,
+					is_string($func)?$parts[1]:$this->stringify()));
 			if ($parts[2]=='->')
 				$parts[1]=is_subclass_of($parts[1],'Prefab')?
 					call_user_func($parts[1].'::instance'):
-					new $parts[1];
+					new $parts[1]($this);
 			$func=array($parts[1],$parts[3]);
 		}
-		if (!is_callable($func) && $hooks=='beforeroute,afterroute')
+		if (!is_callable($func))
 			// No route handler
-			$this->error(404);
+			if ($hooks=='beforeroute,afterroute')
+				$this->error(405);
+			else
+				user_error(sprintf(self::E_Method,
+					is_string($func)?$func:$this->stringify($func)));
 		$obj=FALSE;
 		if (is_array($func)) {
 			$hooks=$this->split($hooks);
@@ -1341,42 +1427,39 @@ final class Base {
 	function config($file) {
 		preg_match_all(
 			'/(?<=^|\n)(?:'.
-			'(?:;[^\n]*)|(?:<\?php.+?\?>?)|'.
-			'(?:\[(.+?)\])|'.
-			'(.+?)\h*=\h*'.
-			'((?:\\\\\h*\r?\n|.+?)*)'.
+				'\[(?<section>.+?)\]|'.
+				'(?<lval>[^\h\r\n;].+?)\h*=\h*'.
+				'(?<rval>(?:\\\\\h*\r?\n|.+?)*)'.
 			')(?=\r?\n|$)/',
 			$this->read($file),$matches,PREG_SET_ORDER);
 		if ($matches) {
 			$sec='globals';
 			foreach ($matches as $match) {
-				if (count($match)<2)
-					continue;
-				if ($match[1])
-					$sec=$match[1];
-				elseif (in_array($sec,array('routes','maps'))) {
+				if ($match['section'])
+					$sec=$match['section'];
+				elseif (in_array($sec,array('routes','maps','redirects'))) {
 					call_user_func_array(
 						array($this,rtrim($sec,'s')),
-						array_merge(array($match[2]),str_getcsv($match[3])));
+						array_merge(array($match['lval']),
+							str_getcsv($match['rval'])));
 				}
 				else {
 					$args=array_map(
 						function($val) {
-							$quote=(isset($val[0]) && $val[0]=="\x00");
-							$val=trim($val);
-							if (!$quote && is_numeric($val))
+							if (is_numeric($val))
 								return $val+0;
+							$val=ltrim($val);
 							if (preg_match('/^\w+$/i',$val) && defined($val))
 								return constant($val);
 							return preg_replace('/\\\\\h*(\r?\n)/','\1',$val);
 						},
 						// Mark quoted strings with 0x00 whitespace
-						str_getcsv(preg_replace(
-							'/(")(.+?)\1/',"\\1\x00\\2\\1",$match[3]))
+						str_getcsv(preg_replace('/(?<!\\\\)(")(.*?)\1/',
+							"\\1\x00\\2\\1",$match['rval']))
 					);
 					call_user_func_array(array($this,'set'),
 						array_merge(
-							array($match[2]),
+							array($match['lval']),
 							count($args)>1?array($args):$args));
 				}
 			}
@@ -1415,7 +1498,7 @@ final class Base {
 	*	@param $lf bool
 	**/
 	function read($file,$lf=FALSE) {
-		$out=file_get_contents($file);
+		$out=@file_get_contents($file);
 		return $lf?preg_replace('/\r\n|\r/',"\n",$out):$out;
 	}
 
@@ -1463,7 +1546,7 @@ final class Base {
 	*	@param $expr mixed
 	**/
 	function dump($expr) {
-		echo $this->highlight($this->stringify($expr,$this->hive['DEBUG']>2));
+		echo $this->highlight($this->stringify($expr));
 	}
 
 	/**
@@ -1498,24 +1581,14 @@ final class Base {
 	**/
 	function unload($cwd) {
 		chdir($cwd);
-		@session_commit();
+		if (!$error=error_get_last())
+			@session_commit();
 		$handler=$this->hive['UNLOAD'];
 		if ((!$handler || $this->call($handler,$this)===FALSE) &&
-			($error=error_get_last()) && in_array($error['type'],
+			$error && in_array($error['type'],
 			array(E_ERROR,E_PARSE,E_CORE_ERROR,E_COMPILE_ERROR)))
 			// Fatal error detected
-			$this->error(500,sprintf(self::E_Fatal,$error['message']),
-				array($error));
-	}
-
-	/**
-	*	Return class instance
-	*	@return object
-	**/
-	static function instance() {
-		if (!Registry::exists($class=__CLASS__))
-			Registry::set($class,new $class);
-		return Registry::get($class);
+			$this->error(500,sprintf(self::E_Fatal,$error['message']));
 	}
 
 	//! Prohibit cloning
@@ -1523,7 +1596,7 @@ final class Base {
 	}
 
 	//! Bootstrap
-	private function __construct() {
+	function __construct() {
 		// Managed directives
 		ini_set('default_charset',$charset='UTF-8');
 		if (extension_loaded('mbstring'))
@@ -1544,7 +1617,7 @@ final class Base {
 		set_error_handler(
 			function($code,$text) use($fw) {
 				if (error_reporting())
-					throw new ErrorException($text,$code);
+					$fw->error(500,$text);
 			}
 		);
 		if (!isset($_SERVER['SERVER_NAME']))
@@ -1571,9 +1644,18 @@ final class Base {
 		$scheme=isset($_SERVER['HTTPS']) && $_SERVER['HTTPS']=='on' ||
 			isset($headers['X-Forwarded-Proto']) &&
 			$headers['X-Forwarded-Proto']=='https'?'https':'http';
+		if (function_exists('apache_setenv')) {
+			// Work around Apache pre-2.4 VirtualDocumentRoot bug
+			$_SERVER['DOCUMENT_ROOT']=str_replace($_SERVER['SCRIPT_NAME'],'',
+				$_SERVER['SCRIPT_FILENAME']);
+			apache_setenv("DOCUMENT_ROOT",$_SERVER['DOCUMENT_ROOT']);
+		}
+		$_SERVER['DOCUMENT_ROOT']=realpath($_SERVER['DOCUMENT_ROOT']);
 		$base='';
 		if (PHP_SAPI!='cli')
-			$base=rtrim(dirname($_SERVER['SCRIPT_NAME']),'/');
+			$base=implode('/',array_map('urlencode',
+				explode('/',rtrim($this->fixslashes(
+					dirname($_SERVER['SCRIPT_NAME'])),'/'))));
 		$path=preg_replace('/^'.preg_quote($base,'/').'/','',
 			parse_url($_SERVER['REQUEST_URI'],PHP_URL_PATH));
 		call_user_func_array('session_set_cookie_params',
@@ -1587,13 +1669,6 @@ final class Base {
 				'httponly'=>TRUE
 			)
 		);
-		if (function_exists('apache_setenv')) {
-			// Work around Apache pre-2.4 VirtualDocumentRoot bug
-			$_SERVER['DOCUMENT_ROOT']=str_replace($_SERVER['SCRIPT_NAME'],'',
-				$_SERVER['SCRIPT_FILENAME']);
-			apache_setenv("DOCUMENT_ROOT",$_SERVER['DOCUMENT_ROOT']);
-		}
-		$_SERVER['DOCUMENT_ROOT']=realpath($_SERVER['DOCUMENT_ROOT']);
 		// Default configuration
 		$this->hive=array(
 			'AGENT'=>isset($headers['X-Operamini-Phone-UA'])?
@@ -1607,12 +1682,14 @@ final class Base {
 			'ALIASES'=>array(),
 			'AUTOLOAD'=>'./',
 			'BASE'=>$base,
+			'BITMASK'=>ENT_COMPAT,
 			'BODY'=>NULL,
 			'CACHE'=>FALSE,
 			'CASELESS'=>TRUE,
 			'DEBUG'=>0,
 			'DIACRITICS'=>array(),
 			'DNSBL'=>'',
+			'EMOJI'=>array(),
 			'ENCODING'=>$charset,
 			'ERROR'=>NULL,
 			'ESCAPE'=>TRUE,
@@ -1660,7 +1737,8 @@ final class Base {
 			'UPLOADS'=>'./',
 			'URI'=>&$_SERVER['REQUEST_URI'],
 			'VERB'=>&$_SERVER['REQUEST_METHOD'],
-			'VERSION'=>self::VERSION
+			'VERSION'=>self::VERSION,
+			'XFRAME'=>'SAMEORIGIN'
 		);
 		if (PHP_SAPI=='cli-server' &&
 			preg_match('/^'.preg_quote($base,'/').'$/',$this->hive['URI']))
@@ -1689,25 +1767,6 @@ final class Base {
 
 }
 
-//! Prefab for classes with constructors and static factory methods
-abstract class Prefab {
-
-	/**
-	*	Return class instance
-	*	@return object
-	**/
-	static function instance() {
-		if (!Registry::exists($class=get_called_class())) {
-			$ref=new Reflectionclass($class);
-			$args=func_get_args();
-			Registry::set($class,
-				$args?$ref->newinstanceargs($args):new $class);
-		}
-		return Registry::get($class);
-	}
-
-}
-
 //! Cache engine
 class Cache extends Prefab {
 
@@ -1716,12 +1775,12 @@ class Cache extends Prefab {
 		$dsn,
 		//! Prefix for cache entries
 		$prefix,
-		//! MemCache object
+		//! MemCache or Redis object
 		$ref;
 
 	/**
 	*	Return timestamp and TTL of cache entry or FALSE if not found
-	*	@return float|FALSE
+	*	@return array|FALSE
 	*	@param $key string
 	*	@param $val mixed
 	**/
@@ -1736,6 +1795,9 @@ class Cache extends Prefab {
 			case 'apcu':
 				$raw=apc_fetch($ndx);
 				break;
+			case 'redis':
+				$raw=$this->ref->get($ndx);
+				break;
 			case 'memcache':
 				$raw=memcache_get($this->ref,$ndx);
 				break;
@@ -1746,14 +1808,14 @@ class Cache extends Prefab {
 				$raw=xcache_get($ndx);
 				break;
 			case 'folder':
-				if (is_file($file=$parts[1].$ndx))
-					$raw=$fw->read($file);
+				$raw=$fw->read($parts[1].$ndx);
 				break;
 		}
 		if (!empty($raw)) {
 			list($val,$time,$ttl)=(array)$fw->unserialize($raw);
 			if ($ttl===0 || $time+$ttl>microtime(TRUE))
 				return array($time,$ttl);
+			$val=null;
 			$this->clear($key);
 		}
 		return FALSE;
@@ -1780,6 +1842,8 @@ class Cache extends Prefab {
 			case 'apc':
 			case 'apcu':
 				return apc_store($ndx,$data,$ttl);
+			case 'redis':
+				return $this->ref->set($ndx,$data,array('ex'=>$ttl));
 			case 'memcache':
 				return memcache_set($this->ref,$ndx,$data,0,$ttl);
 			case 'wincache':
@@ -1815,6 +1879,8 @@ class Cache extends Prefab {
 			case 'apc':
 			case 'apcu':
 				return apc_delete($ndx);
+			case 'redis':
+				return $this->ref->del($ndx);
 			case 'memcache':
 				return memcache_delete($this->ref,$ndx);
 			case 'wincache':
@@ -1822,7 +1888,7 @@ class Cache extends Prefab {
 			case 'xcache':
 				return xcache_unset($ndx);
 			case 'folder':
-				return is_file($file=$parts[1].$ndx) && @unlink($file);
+				return @unlink($parts[1].$ndx);
 		}
 		return FALSE;
 	}
@@ -1851,6 +1917,15 @@ class Cache extends Prefab {
 						$item['mtime']+$lifetime<time())
 						apc_delete($item[$key]);
 				return TRUE;
+			case 'redis':
+				$fw=Base::instance();
+				$keys=$this->ref->keys($this->prefix.'.*'.$suffix);
+				foreach($keys as $key) {
+					$val=$fw->unserialize($this->ref->get($key));
+					if ($val[1]+$lifetime<time())
+						$this->ref->del($key);
+				}
+				return TRUE;
 			case 'memcache':
 				foreach (memcache_get_extended_stats(
 					$this->ref,'slabs') as $slabs)
@@ -1892,7 +1967,19 @@ class Cache extends Prefab {
 	function load($dsn) {
 		$fw=Base::instance();
 		if ($dsn=trim($dsn)) {
-			if (preg_match('/^memcache=(.+)/',$dsn,$parts) &&
+			if (preg_match('/^redis=(.+)/',$dsn,$parts) &&
+				extension_loaded('redis')) {
+				$port=6379;
+				$parts=explode(':',$parts[1],2);
+				if (count($parts)>1)
+					list($host,$port)=$parts;
+				else
+					$host=$parts[0];
+				$this->ref=new Redis;
+				if(!$this->ref->connect($host,$port,2))
+					$this->ref=NULL;
+			}
+			elseif (preg_match('/^memcache=(.+)/',$dsn,$parts) &&
 				extension_loaded('memcache'))
 				foreach ($fw->split($parts[1]) as $server) {
 					$port=11211;
@@ -1940,42 +2027,18 @@ class View extends Prefab {
 		//! Template file
 		$view;
 
-	/**
-	*	Attempt to clone object
-	*	@return object
-	*	@return $arg object
-	**/
-	function dupe($arg) {
-		if (method_exists('ReflectionClass','iscloneable')) {
-			$ref=new ReflectionClass($arg);
-			if ($ref->iscloneable())
-				$arg=clone($arg);
-		}
-		return $arg;
-	}
-
 	/**
 	*	Encode characters to equivalent HTML entities
 	*	@return string
 	*	@param $arg mixed
 	**/
 	function esc($arg) {
-		if (is_array($arg)) {
-			$tmp=array();
-			foreach ($arg as $key=>$val)
-				$tmp[$key]=$this->esc($val);
-			return $tmp;
-		}
-		if (is_object($arg)) {
-			$obj=$this->dupe($arg);
-			foreach (get_object_vars($obj) as $key=>$val)
-				$obj->$key=$this->esc($val);
-			return $obj;
-		}
-		$arg=unserialize(serialize($arg));
-		if (is_string($arg))
-			$arg=\Base::instance()->encode($arg);
-		return $arg;
+		$fw=Base::instance();
+		return $fw->recursive($arg,
+			function($val) use($fw) {
+				return is_string($val)?$fw->encode($val):$val;
+			}
+		);
 	}
 
 	/**
@@ -1984,22 +2047,12 @@ class View extends Prefab {
 	*	@param $arg mixed
 	**/
 	function raw($arg) {
-		if (is_array($arg)) {
-			$tmp=array();
-			foreach ($arg as $key=>$val)
-				$tmp[$key]=$this->raw($val);
-			return $tmp;
-		}
-		if (is_object($arg)) {
-			$obj=$this->dupe($arg);
-			foreach (get_object_vars($obj) as $key=>$val)
-				$obj->$key=$this->raw($val);
-			return $obj;
-		}
-		$arg=unserialize(serialize($arg));
-		if (is_string($arg))
-			$arg=\Base::instance()->decode($arg);
-		return $arg;
+		$fw=Base::instance();
+		return $fw->recursive($arg,
+			function($val) use($fw) {
+				return is_string($val)?$fw->decode($val):$val;
+			}
+		);
 	}
 
 	/**
@@ -2013,6 +2066,8 @@ class View extends Prefab {
 			$hive=$fw->hive();
 		if ($fw->get('ESCAPE'))
 			$hive=$this->esc($hive);
+		if (isset($hive['ALIASES']))
+			$hive['ALIASES']=$fw->build($hive['ALIASES']);
 		extract($hive);
 		unset($fw);
 		unset($hive);
@@ -2040,7 +2095,7 @@ class View extends Prefab {
 				if (isset($_COOKIE[session_name()]))
 					@session_start();
 				$fw->sync('SESSION');
-				if (PHP_SAPI!='cli')
+				if ($mime && PHP_SAPI!='cli')
 					header('Content-Type: '.$mime.'; '.
 						'charset='.$fw->get('ENCODING'));
 				$data=$this->sandbox($hive);
@@ -2066,31 +2121,8 @@ class Preview extends View {
 	*	@param $str string
 	**/
 	function token($str) {
-		$self=$this;
-		$str=preg_replace_callback(
-			'/(?<!\w)@(\w(?:[\w\.\[\]]|\->|::)*)/',
-			function($var) use($self) {
-				// Convert from JS dot notation to PHP array notation
-				return '$'.preg_replace_callback(
-					'/(\.\w+)|\[((?:[^\[\]]*|(?R))*)\]/',
-					function($expr) use($self) {
-						$fw=Base::instance();
-						return
-							'['.
-							($expr[1]?
-								$fw->stringify(substr($expr[1],1)):
-								(preg_match('/^\w+/',
-									$mix=$self->token($expr[2]))?
-									$fw->stringify($mix):
-									$mix)).
-							']';
-					},
-					$var[1]
-				);
-			},
-			$str
-		);
-		return trim(preg_replace('/\{\{(.+?)\}\}/s',trim('\1'),$str));
+		return trim(preg_replace('/\{\{(.+?)\}\}/s',trim('\1'),
+			Base::instance()->compile($str)));
 	}
 
 	/**
@@ -2101,18 +2133,21 @@ class Preview extends View {
 	protected function build($node) {
 		$self=$this;
 		return preg_replace_callback(
-			'/\{\{(.+?)\}\}/s',
+			'/\{\{(.+?)\}\}(\n)?/s',
 			function($expr) use($self) {
 				$str=trim($self->token($expr[1]));
-				if (preg_match('/^(.+?)\h*\|\h*(raw|esc|format)$/',
-					$str,$parts))
-					$str=(($parts[2]=='format')?
-						'\Base::instance()':'$this').'->'.$parts[2].
-						'('.$parts[1].')';
-				return '<?php echo '.$str.'; ?>';
+				if (preg_match('/^([^|]+?)\h*\|(\h*\w+(?:\h*[,;]\h*\w+)*)/',
+					$str,$parts)) {
+					$str=$parts[1];
+					foreach (Base::instance()->split($parts[2]) as $func)
+						$str=(($func=='format')?'\Base::instance()':'$this').
+							'->'.$func.'('.$str.')';
+				}
+				return '<?php echo '.$str.'; ?>'.
+					(isset($expr[2])?"\n\n":'');
 			},
 			preg_replace_callback(
-				'/\{\{?~(.+?)~\}?\}/s',
+				'/\{~(.+?)~\}/s',
 				function($expr) use($self) {
 					return '<?php '.$self->token($expr[1]).' ?>';
 				},
@@ -2161,7 +2196,7 @@ class Preview extends View {
 					// Remove PHP code and comments
 					$text=preg_replace(
 						'/(?<!["\'])\h*<\?(?:php|\s*=).+?\?>\h*(?!["\'])|'.
-						'\{\{\*.+?\*\}\}|\{\*.+?\*\}/is','',
+						'\{\*.+?\*\}/is','',
 						$fw->read($view));
 					if (method_exists($this,'parse'))
 						$text=$this->parse($text);
@@ -2170,7 +2205,7 @@ class Preview extends View {
 				if (isset($_COOKIE[session_name()]))
 					@session_start();
 				$fw->sync('SESSION');
-				if (PHP_SAPI!='cli')
+				if ($mime && PHP_SAPI!='cli')
 					header('Content-Type: '.($this->mime=$mime).'; '.
 						'charset='.$fw->get('ENCODING'));
 				$data=$this->sandbox($hive);
