@@ -56,10 +56,7 @@ selfoss.events.entries = function(e) {
             
             // set events for fullscreen
             selfoss.events.entriesToolbar(fullscreen);
-            
-            // set color of all tags by background color
-            fullscreen.find('.entry-tags-tag').colorByBrightness();
-    
+
             // set events for closing fullscreen
             fullscreen.find('.entry, .entry-close').click(function(e) {
                 if(e.target.tagName.toLowerCase()=="a")
@@ -91,6 +88,9 @@ selfoss.events.entries = function(e) {
                 if(autoMarkAsRead) {
                     parent.find('.entry-unread').click();
                 }
+
+                // setup fancyBox image viewer
+                selfoss.setupFancyBox(content, parent.attr('id').substr(5));
             }
             
             // load images not on mobile devices
@@ -106,7 +106,8 @@ selfoss.events.entries = function(e) {
     
     // scroll load more
     $(window).unbind('scroll').scroll(function() {
-        if($('#content').is(':visible')==false)
+        if($('#config').data('auto_stream_more') == 0 ||
+           $('#content').is(':visible')==false)
             return;
     
         var content = $('#content');
@@ -116,6 +117,13 @@ selfoss.events.entries = function(e) {
             $('.stream-more').click();
     });
     
+    $('.mark-these-read').unbind('click').click(function () {
+        $('#nav-mark').click();
+        // hide nav on smartphone
+        if(selfoss.isSmartphone())
+            $('#nav-mobile-settings').click();
+    });
+
     // more
     $('.stream-more').unbind('click').click(function () {
         var streamMore = $(this);
@@ -128,6 +136,7 @@ selfoss.events.entries = function(e) {
             dataType: 'json',
             data: selfoss.filter,
             success: function(data) {
+                streamMore.siblings('.mark-these-read').remove();
                 $('.stream-more').replaceWith(data.entries);
                 selfoss.events.entries();
             },
@@ -138,9 +147,6 @@ selfoss.events.entries = function(e) {
         });
     });
     
-    // set color of all tags by background color
-    $('.entry-tags-tag').colorByBrightness();
-    
     // click a tag
     if(selfoss.isSmartphone()==false) {
         $('.entry-tags-tag').unbind('click').click(function() {
